/*
Exhaust Particle Script
by Strife

Based on particle code by Lach
*/

local SPEED = 3 // speed at which particles rise
local LIFETIME = TICRATE // maximum lifetime for particles
local MIN_DELAY = 1 // minimum amout of time before new particle appears
local MAX_DELAY = 7 // ditto but maximum
local FLAME_DEADZONE = 10 // flames will not appear if the boost/drift timer is below this

// Skins that are "kartless" and will not emit particles
local SKIN_NONE = {
	"srb2sonic",
	"ksonic",
	"ktails",
	"kknuckles",
	"nanachi"
}

// Skins using the Crash Team Racing kart by Plom
local SKIN_CTR = {
	"crashbandicoot",
	"cocobandicoot",
	"dingodile",
	"polar",
	"pentapenguin",
	"pinstripe",
	"ntropy",
	"ntrance",
	"nsisabella",
	"nsami",
	"nsmegumi",
	"nsliz",
	"spyro",
	"kingpauliii",
	"ctrspyro",
	"ngin",
	"fakecrash",
	"smallnorm",
	"tawnabandicoot",
	"hereiam"
}

-- Exhaust states
freeslot("MT_EXHS", 
	"S_EXHS1", "S_EXHS2", "S_EXHS3", "S_EXHS4", "S_EXHS5",
	"SPR_EXHS")
states[S_EXHS1] = {SPR_EXHS, FF_TRANS50|A, 1, nil, 0, 0, S_EXHS2}
states[S_EXHS2] = {SPR_EXHS, FF_TRANS60|B, 1, nil, 0, 0, S_EXHS3}
states[S_EXHS3] = {SPR_EXHS, FF_TRANS70|C, 1, nil, 0, 0, S_EXHS4}
states[S_EXHS4] = {SPR_EXHS, FF_TRANS80|D, 1, nil, 0, 0, S_EXHS5}
states[S_EXHS5] = {SPR_EXHS, FF_TRANS90|E, 1, nil, 0, 0, 0}

-- Smoke states
freeslot("MT_EXH2", 
	"S_EXH21", "S_EXH22", "S_EXH23", "S_EXH24", "S_EXH25",
	"SPR_SMOK")
states[S_EXH21] = {SPR_SMOK, FF_TRANS10|A, 2, nil, 0, 0, S_EXH22}
states[S_EXH22] = {SPR_SMOK, FF_TRANS30|B, 2, nil, 0, 0, S_EXH23}
states[S_EXH23] = {SPR_SMOK, FF_TRANS50|C, 2, nil, 0, 0, S_EXH24}
states[S_EXH24] = {SPR_SMOK, FF_TRANS70|D, 2, nil, 0, 0, S_EXH25}
states[S_EXH25] = {SPR_SMOK, FF_TRANS90|E, 2, nil, 0, 0, 0}

-- Flame states
freeslot("MT_EXHF", 
	"S_EXHF1", "S_EXHF2", "S_EXHF3",
	"SPR_EXHF")
states[S_EXHF1] = {SPR_EXHF, FF_FULLBRIGHT|A, 1, nil, 0, 0, S_EXHF2}
states[S_EXHF2] = {SPR_EXHF, FF_FULLBRIGHT|B, 1, nil, 0, 0, S_EXHF3}
states[S_EXHF3] = {SPR_EXHF, FF_FULLBRIGHT|C, 1, nil, 0, 0, 0}

-- Vanilla boost effect removal
addHook("MobjThinker", function(mo)
    if (mo.target and mo.target.player) then
        mo.flametimer = ($ or 0) + 1
        if mo.flametimer > 15 then
            P_RemoveMobj(mo)
        end
    end
end, MT_BOOSTFLAME)

addHook("ThinkFrame", do
	for player in players.iterate
		local mo = player.mo
		
		if not mo
			continue
		end
		
		local frame = mo.frame & FF_FRAMEMASK
		local pipedepth = -18 // how far back the exhaust pipes are
		local pipewidth = 8 // how far apart the pipes are
		local pipeheight = 14 // how high from the ground the pipes are
		
		// Check for CTR skins
		for i = #SKIN_CTR, 1, -1
			if (mo.skin == SKIN_CTR[i])
				pipedepth = -21
				pipewidth = 8
				pipeheight = 22
			end
		end
		
		// Check for kartless skins
		for i = #SKIN_CTR, 1, -1
			if (mo.skin == SKIN_NONE[i])
				mo.leftexhaustdelay = MAX_DELAY
				mo.rightexhaustdelay = MAX_DELAY
			end
		end
		
		pipedepth = $ * mo.scale/FRACUNIT
		pipewidth = $ * mo.scale/FRACUNIT
		pipeheight = $ * mo.scale/FRACUNIT
		
		local ang = player.frameangle
		
		if frame == M or frame == N
			ang = $ + ANGLE_45
		elseif frame == O or frame == P
			ang = $ - ANGLE_45
		end

		// Left Particle Spawning
		if not mo.leftexhaust
			mo.leftexhaust = {}
		end
		
		if not mo.leftexhaustdelay
			local radius = mo.radius >> FRACBITS
			local height = mo.height >> FRACBITS
			
			local part = P_SpawnMobj(mo.x + cos(ang) * pipedepth + sin(ang) * -pipewidth, mo.y + sin(ang) * pipedepth + cos(ang) * pipewidth, mo.z + pipeheight*FRACUNIT, MT_PARTICLE)
			part.scale = mo.scale
			part.flags2 = mo.flags2
			part.eflags = mo.eflags
			
			// Use smoke sprites while boosting, and spawn a second sprite for flames
			if(player.kartstuff[k_driftboost] or player.kartstuff[k_startboost] or player.kartstuff[k_sneakertimer])
				part.state = S_EXH21
				part.zdist = FixedMul(-1, part.scale)
				// Flame sprites
				if (player.kartstuff[k_driftboost] > FLAME_DEADZONE or player.kartstuff[k_startboost] > FLAME_DEADZONE or player.kartstuff[k_sneakertimer] > FLAME_DEADZONE)
					local part2 = P_SpawnMobj(mo.x + cos(ang) * pipedepth + sin(ang) * -pipewidth, mo.y + sin(ang) * pipedepth + cos(ang) * pipewidth, mo.z + pipeheight*FRACUNIT, MT_PARTICLE)
					part2.scale = mo.scale
					part2.flags2 = mo.flags2
					part2.eflags = mo.eflags
					part2.state = S_EXHF1
					part2.fuse = LIFETIME
					part2.zdist = 0
					table.insert(mo.leftexhaust, part2)
				end
			else
				part.state = S_EXHS1
				part.zdist = 0
			end
			part.fuse = LIFETIME
			
			table.insert(mo.leftexhaust, part)
			
			mo.leftexhaustdelay = P_RandomRange(MIN_DELAY, MAX_DELAY)
		end
		mo.leftexhaustdelay = $ - 1
		
		// Left Particle Movement
		for i = #mo.leftexhaust, 1, -1
			local part = mo.leftexhaust[i]
			if not (part and part.valid)
				table.remove(mo.leftexhaust, i)
				continue
			end
			
			part.zdist = $ + FixedMul(SPEED, part.scale)
			// Normal gravity
			if (P_MobjFlip(part) > 0)
				P_TeleportMove(part, mo.x + cos(ang) * (pipedepth - part.zdist) + sin(ang) * -pipewidth, mo.y + sin(ang) * (pipedepth - part.zdist) + cos(ang) * pipewidth, mo.z + (pipeheight + part.zdist)*FRACUNIT)
			// Reverse gravity
			else
				P_TeleportMove(part, mo.x + cos(ang) * (pipedepth - part.zdist) + sin(ang) * -pipewidth, mo.y + sin(ang) * (pipedepth - part.zdist) + cos(ang) * pipewidth, mo.z + mo.height - (pipeheight + 4 + part.zdist)*FRACUNIT)
			end
		end
		
		// Right Particle Spawning
		if not mo.rightexhaust
			mo.rightexhaust = {}
		end
		
		if not mo.rightexhaustdelay
			local radius = mo.radius >> FRACBITS
			local height = mo.height >> FRACBITS
			
			local part = P_SpawnMobj(mo.x + cos(ang) * pipedepth + sin(ang) * pipewidth, mo.y + sin(ang) * pipedepth + cos(ang) * -pipewidth, mo.z + pipeheight*FRACUNIT, MT_PARTICLE)
			part.scale = mo.scale
			part.flags2 = mo.flags2
			part.eflags = mo.eflags
			
			// Use smoke sprites while boosting, and spawn a second sprite for flames
			if(player.kartstuff[k_driftboost] or player.kartstuff[k_startboost] or player.kartstuff[k_sneakertimer])
				part.state = S_EXH21
				part.zdist = FixedMul(-1, part.scale)
				// Flame sprites
				if (player.kartstuff[k_driftboost] > FLAME_DEADZONE or player.kartstuff[k_startboost] > FLAME_DEADZONE or player.kartstuff[k_sneakertimer] > FLAME_DEADZONE)
					local part2 = P_SpawnMobj(mo.x + cos(ang) * pipedepth + sin(ang) * -pipewidth, mo.y + sin(ang) * pipedepth + cos(ang) * pipewidth, mo.z + pipeheight*FRACUNIT, MT_PARTICLE)
					part2.scale = mo.scale
					part2.flags2 = mo.flags2
					part2.eflags = mo.eflags
					part2.state = S_EXHF1
					part2.fuse = LIFETIME
					part2.zdist = 0
					table.insert(mo.rightexhaust, part2)
				end
			else
				part.state = S_EXHS1
				part.zdist = 0
			end
			part.fuse = LIFETIME
			
			table.insert(mo.rightexhaust, part)
			
			mo.rightexhaustdelay = P_RandomRange(MIN_DELAY, MAX_DELAY)
		end
		mo.rightexhaustdelay = $ - 1
		
		// Right Particle Movement
		for i = #mo.rightexhaust, 1, -1
			local part = mo.rightexhaust[i]
			if not (part and part.valid)
				table.remove(mo.rightexhaust, i)
				continue
			end
			
			part.zdist = $ + FixedMul(SPEED, part.scale)
			// Normal gravity
			if (P_MobjFlip(part) > 0)
				P_TeleportMove(part, mo.x + cos(ang) * (pipedepth - part.zdist) + sin(ang) * pipewidth, mo.y + sin(ang) * (pipedepth - part.zdist) + cos(ang) * -pipewidth, mo.z + (pipeheight + part.zdist)*FRACUNIT)
			// Reverse gravity
			else
				P_TeleportMove(part, mo.x + cos(ang) * (pipedepth - part.zdist) + sin(ang) * pipewidth, mo.y + sin(ang) * (pipedepth - part.zdist) + cos(ang) * -pipewidth, mo.z + mo.height - (pipeheight + 4 + part.zdist)*FRACUNIT)
			end
		end
		
	end
end)

rawset(_G, "EXHAUST_SetType",
	function(p, v) -- Allows custom skins to choose an offset type
		if (v == "NONE")
			table.insert(SKIN_NONE, p)
		end
		if (v == "CTR")
			table.insert(SKIN_CTR, p)
		end
		print("EXHUAST: Added " .. p .. "to " .. v)
	end
)