local hfp1a = {"","","",""} -- p1 names
local hfp2a = {"","","",""} -- p2 names
local hfp1c = {0,0,0,0} -- p1 colors
local hfp2c = {0,0,0,0} -- p2 colors
local hfia = {"","","",""} -- inflictor names
local hfsta = {0,0,0,0} -- start tic

local showhitfeed = CV_RegisterVar({ -- lets player disable hitfeed rendering for performance improvements
	name = "showhitfeed",
	defaultvalue = "on",
	possiblevalue = CV_OnOff
})

local hitfeeddiscord = CV_RegisterVar({ -- if on, throws stuff on log so it can be read by other programs; non-dedicated servers should disable
	name = "hitfeeddiscord",
	defaultvalue = "on",
    flags = CV_NETVAR, -- only affects server so regular players shouldnt worry about it
	possiblevalue = CV_OnOff
})

local hitfeednamecolor = CV_RegisterVar({ -- enables/disables name coloring (defaults to white if false)
	name = "hitfeednamecolor",
	defaultvalue = "on",
	possiblevalue = CV_OnOff
})

local function hasVictim(index)
    return hfp2a[index] ~= "" and hfp2a[index] ~= nil
end

local function getNameColor(c)
    if hitfeednamecolor.value == 0 then
        return 0
    end
    local color_sets = { --stolen from regular hitfeed, sorry
	{SKINCOLOR_WHITE, SKINCOLOR_SILVER, SKINCOLOR_SLATE}, -- 1: No colour mapping
	{SKINCOLOR_GREY, SKINCOLOR_NICKEL, SKINCOLOR_BLACK, SKINCOLOR_JET}, -- 2: V_GREYMAP
	{SKINCOLOR_SEPIA, SKINCOLOR_BEIGE, SKINCOLOR_BROWN, SKINCOLOR_LEATHER, SKINCOLOR_RUST}, -- 3: V_BROWNMAP
	{SKINCOLOR_FAIRY, SKINCOLOR_SALMON, SKINCOLOR_PINK, SKINCOLOR_ROSE, SKINCOLOR_BRICK, SKINCOLOR_BUBBLEGUM, SKINCOLOR_LILAC}, -- 4: V_PINKMAP
	{SKINCOLOR_RUBY, SKINCOLOR_RASPBERRY, SKINCOLOR_CHERRY, SKINCOLOR_RED, SKINCOLOR_CRIMSON, SKINCOLOR_MAROON, SKINCOLOR_FLAME, SKINCOLOR_SCARLET, SKINCOLOR_KETCHUP}, -- 5: V_REDMAP
	{SKINCOLOR_DAWN, SKINCOLOR_SUNSET, SKINCOLOR_CREAMSICLE, SKINCOLOR_ORANGE, SKINCOLOR_PUMPKIN, SKINCOLOR_ROSEWOOD, SKINCOLOR_BURGUNDY, SKINCOLOR_TANGERINE}, -- 6: V_ORANGEMAP
	{SKINCOLOR_PEACH, SKINCOLOR_CARAMEL, SKINCOLOR_CREAM}, -- 7: V_PEACHMAP
	{SKINCOLOR_GOLD, SKINCOLOR_ROYAL, SKINCOLOR_BRONZE, SKINCOLOR_COPPER}, -- 8: V_GOLDMAP
	{SKINCOLOR_POPCORN, SKINCOLOR_YELLOW, SKINCOLOR_MUSTARD, SKINCOLOR_OLIVE}, -- 9: V_YELLOWMAP
	{SKINCOLOR_VOMIT, SKINCOLOR_GARDEN, SKINCOLOR_TEA, SKINCOLOR_PISTACHIO}, -- 10: V_TEAMAP
	{SKINCOLOR_LIME, SKINCOLOR_HANDHELD, SKINCOLOR_MOSS, SKINCOLOR_CAMOUFLAGE, SKINCOLOR_ROBOHOOD, SKINCOLOR_MINT, SKINCOLOR_GREEN, SKINCOLOR_PINETREE, SKINCOLOR_EMERALD, SKINCOLOR_SWAMP, SKINCOLOR_DREAM, SKINCOLOR_PLAGUE, SKINCOLOR_ALGAE}, -- 11: V_GREENMAP
	{SKINCOLOR_CARIBBEAN, SKINCOLOR_AQUA, SKINCOLOR_TEAL, SKINCOLOR_CYAN, SKINCOLOR_JAWZ, SKINCOLOR_CERULEAN, SKINCOLOR_NAVY, SKINCOLOR_SAPPHIRE}, -- 12: V_SKYMAP
	{SKINCOLOR_PLATINUM, SKINCOLOR_STEEL}, -- 13: V_STEELMAP
	{SKINCOLOR_PERIWINKLE, SKINCOLOR_BLUE, SKINCOLOR_BLUEBERRY, SKINCOLOR_NOVA}, -- 14: V_BLUEMAP
	{SKINCOLOR_ULTRAVIOLET, SKINCOLOR_PURPLE, SKINCOLOR_FUCHSIA}, -- 15: V_PURPLEMAP
	{SKINCOLOR_PASTEL, SKINCOLOR_MOONSLAM, SKINCOLOR_DUSK, SKINCOLOR_TOXIC, SKINCOLOR_MAUVE, SKINCOLOR_LAVENDER, SKINCOLOR_BYZANTIUM, SKINCOLOR_POMEGRANATE} -- 16: V_LAVENDERMAP
	}
	
	//local maps = {"\x80", "\x86", "\x8e", "\x8d", "\x85", "\x87", "\x8f", "\x8A", "\x82", "\x8b", "\x83", "\x88", "\x8c", "\x84", "\x81", "\x89"}
	local maps = {0, V_GRAYMAP, V_BROWNMAP, V_PINKMAP, V_REDMAP, V_ORANGEMAP, V_PEACHMAP, V_GOLDMAP, V_YELLOWMAP, V_TEAMAP, V_GREENMAP, V_SKYMAP, V_STEELMAP, V_BLUEMAP, V_PURPLEMAP, V_LAVENDERMAP}
	for i = 1, #color_sets
		for j = 1, #color_sets[i]
			local color = color_sets[i][j]
			
			if (color == c)
				return maps[i]
			end
		end
	end
	
	return 0
end

local function calcOpac(startTic, curTic, speed)
    speed = speed or 10
    if curTic <= startTic + speed then
        return 0
    end
    local asdf = (curTic - startTic) / speed
    if asdf < 0 then
        asdf = 0
    elseif asdf > 10 then
        asdf = 10
    end
    return (65536 * asdf)
end

addHook("MapChange", do -- reset values
    hfp1a = {"","","",""}
    hfp2a = {"","","",""}
    hfp1c = {0,0,0,0}
    hfp2c = {0,0,0,0}
    hfia = {"","","",""}
    hfsta = {0,0,0,0}
end)

local function getDiscordEmoji(sprite)
    if(sprite == "K_ISGROW") return "<:KISGROW:873606763646173234>" end
    if(sprite == "K_ISSINK") return "<:KISSINK:873606763704885288>" end
    if(sprite == "K_ISBANA") return "<:KISBANA:873606763692318811>" end
    if(sprite == "K_ISORBN") return "<:KISORBN:873606763713273856>" end
    if(sprite == "K_ISJAWZ") return "<:KISJAWZ:873606763843301396>" end
    if(sprite == "K_ISBHOG") return "<:KISBHOG:873606763713273876>" end
    if(sprite == "K_ISSPB")  return "<:KISSPB:873606766225653760>"  end
    if(sprite == "K_ISMINE") return "<:KISMINE:873606763662950400>" end
    if(sprite == "K_ISEGGM") return "<:KISEGGM:873606763637792878>" end
    if(sprite == "K_ISINV1") return "<:KISINV6:873606763780403220>" end
    if(sprite == "K_ISTHNS") return "<:KISTHNS:873606763759423488>" end
    if(sprite == "K_ISRSHE") return "<:KISRSHE:873606763348365394>" end
    if(sprite == "K_ISSHOE") return "<:KISSHOE:873606763264503851>" end
    if(sprite == "K_ISPOGO") return "<:KISPOGO:873606763310632992>" end
    return sprite
end

local function setHitFeedStuff(player, inflictor, source, hitType)
    if(inflictor ~= nil and source ~= nil and source.player ~= nil)
        hfp1a[4],hfp2a[4],hfp1c[4],hfp2c[4],hfia[4],hfsta[4] = hfp1a[3],hfp2a[3],hfp1c[3],hfp2c[3],hfia[3],hfsta[3]
        hfp1a[3],hfp2a[3],hfp1c[3],hfp2c[3],hfia[3],hfsta[3] = hfp1a[2],hfp2a[2],hfp1c[2],hfp2c[2],hfia[2],hfsta[2]
        hfp1a[2],hfp2a[2],hfp1c[2],hfp2c[2],hfia[2],hfsta[2] = hfp1a[1],hfp2a[1],hfp1c[1],hfp2c[1],hfia[1],hfsta[1]
        hfp1a[1] = source.player.name
        hfp2a[1] = player.name
        hfp1c[1] = source.player.skincolor
        hfp2c[1] = player.skincolor
        -- get inflictor name - this bit is partailly based on the original hitfeed
        if(hitType == 2) then hfia[1] = "K_ISGROW"
        elseif(hitType == 4) then hfia[1] = "K_ISSINK"
        else
            local items = {
	        [MT_BANANA] = "K_ISBANA",
	        [MT_BANANA_SHIELD] = "K_ISBANA",
	        [MT_ORBINAUT] = "K_ISORBN",
	        [MT_ORBINAUT_SHIELD] = "K_ISORBN",
	        [MT_JAWZ] = "K_ISJAWZ",
	        [MT_JAWZ_DUD] = "K_ISJAWZ",
	        [MT_JAWZ_SHIELD] = "K_ISJAWZ",
	        [MT_BALLHOG] = "K_ISBHOG",
	        [MT_SPB] = "K_ISSPB",
	        [MT_SPBEXPLOSION] = "K_ISSPB",
	        [MT_MINEEXPLOSION] = "K_ISMINE",
	        [MT_SSMINE] = "K_ISMINE",
	        [MT_SSMINE_SHIELD] = "K_ISMINE",
            [MT_KARMAHITBOX] = "K_ISSPB",
	        }
	        if inflictor.type == MT_SPBEXPLOSION and not inflictor.extravalue1
		        hfia[1] = "K_ISEGGM"
	        elseif items[inflictor.type]
		        hfia[1] = items[inflictor.type]
            elseif (inflictor.player)
	            if (source.player.kartstuff[k_invincibilitytimer])
		            hfia[1] = "K_ISINV1"
	            elseif (inflictor.player.kartstuff[k_curshield] == 1)
		            hfia[1] = "K_ISTHNS"
	            elseif (source.player.kartstuff[k_sneakertimer] or source.player.kartstuff[k_pogospring])
		            if source.player.kartstuff[k_sneakertimer]
			            if source.player.kartstuff[k_rocketsneakertimer]
				            hfia[1] = "K_ISRSHE"
			            else
				            hfia[1] = "K_ISSHOE"
			            end
		            else
			            hfia[1] = "K_ISPOGO"
		            end
	            end
            end
        end
        hfsta[1] = leveltime
        -- if isdedicatedserver
        if hitfeeddiscord.value == 1 then
            local emoji = getDiscordEmoji(hfia[1])
            CONS_Printf(server, "[HITFEED] " + source.player.name +  " " + emoji + " " + player.name)
        end
            -- file = io.openlocal("hitfeed.log", "a")
            -- file:write(tostring(player) ..  "\t" .. tostring(inflictor) .. "\t" .. tostring(source) .. "\t" .. tostring(hitType))
            -- io.close(file)
        -- end 
    end
end

addHook("PlayerSpin", function(player, inflictor, source)
    if showhitfeed.value == 1 or hitfeeddiscord.value == 1
        setHitFeedStuff(player, inflictor, source, 1)
    end
end)
addHook("PlayerSquish", function(player, inflictor, source) -- esmagou
    if showhitfeed.value == 1 or hitfeeddiscord.value == 1
        setHitFeedStuff(player, inflictor, source, 2)
    end
end)
addHook("PlayerExplode", function(player, inflictor, source)
    if showhitfeed.value == 1 or hitfeeddiscord.value == 1
        setHitFeedStuff(player, inflictor, source, 3)
    end
end)
addHook("MobjDamage", do
    if showhitfeed.value == 1 or hitfeeddiscord.value == 1
        setHitFeedStuff(player, inflictor, source, 4)
    end
end) -- literalmente uma pia